#!/usr/local/bin/python -O
"""
   Retrieve MIB variables associated with user specifed SNMP Object IDs
   from arbitrary SNMP agent. Employ parallel algorithm of bulk retrieval.
   The number of simultaneous SNMP sessions is limited by the maximum number
   of file descriptors available to process.

   Since MIB parser is not yet implemented in Python, this script takes and
   reports Object IDs in dotted numeric representation only.

   Written by Ilya Etingof <ilya@glas.net>, 10/2000

"""
import types

# Import PySNMP module
from pysnmp import msession

class snmpbulk (msession.multisession):
    """Fetch a bulk of variables from remote SNMP process
    """
    def __init__ (self):
        """Explicitly call superclass's constructor as it gets
           overloaded by this class constructor and pass a few
           arguments alone.
        """   
        msession.multisession.__init__ (self)

    def run (self, agent, community, objids):
        """Issue a bulk of SNMP GET requests and wait for replies to
           come. The objids argument should be a list of strings where
           each string represents a Object ID in dotted numbers notation
           (e.g. ['.1.3.6.1.4.1.307.3.2.1.1.1.4.1']).
        """   
        # Convert string type Object IDs into numeric representation
        numeric_objids = map (self.str2nums, objids)

        # BER encode SNMP Object IDs to query
        encoded_objids = map (self.encode_oid, numeric_objids)

        # Since we are going to _query_ SNMP agent for Object IDs
        # associated value, there will be no variable values passed to
        # SNMP agent.
        encoded_values = []

        # Walk over supplied Object IDs
        for encoded_objid in encoded_objids:
            # Build and submit complete SNMP message of type 'GETREQUEST',
            # pass it a list of BER encoded Object IDs (only one as we are
            # going to query a number of Object IDs in bulk) to query and
            # an empty list of values associated with these Object IDs
            # (empty list as there is no point to pass any variables values
            # along the SNMP GET request).
            self.submit_request (agent, community, 'GETREQUEST',\
                                 [ encoded_objid ], encoded_values)

        # At this point we get a number of submitted SNMP GET requests. Now,
        # send them out all at once.
        self.dispatch ()

        # At this point we get all the replies from SNMP agents we expected
        # (some may time out). Retrieve a list of tuples (encoded_objid,
        # encoded_value)
        encoded_pairs = self.retrieve ()

        # Parse out encoded objid's and encoded values.
        # WARNING: for sake of simplicity, failed SNMP requests
        # (represented as empty encoded_pairs entries) would
        # be removed by the reduce() function applied below
        # so that nothing will be reported back to user of this script
        # for failed SNMP requests.
        encoded_objids = reduce (lambda x, y: x + y,\
                                 map (lambda x: x[0], encoded_pairs))
        encoded_values = reduce (lambda x, y: x + y,\
                                 map (lambda x: x[1], encoded_pairs))

        # Decode BER encoded Object ID.
        objids = map (self.decode_value, encoded_objids)

        # Decode BER encoded values associated with Object IDs.
        values = map (self.decode_value, encoded_values)
        
        # Return a tuple of two lists holding Object IDs and associated
        # values extracted from SNMP agent replies.
        return (objids, values)

# Run the module if it's invoked for execution
if __name__ == '__main__':
    import sys

    # Make sure we have got enough args
    if len (sys.argv) < 4:
        # Report a usage error
        print 'Usage: %s <snmp-agent> <communuty> <object-id [[object-id] ... ]>' % sys.argv[0]

        # Incomplete command line arguments, exiting
        sys.exit (1)

    # Create an instance of snmpget class
    instance = snmpbulk ()

    # Run snmpbulk against passed Object IDs and expect it to return a list
    # of Object IDs and corresponding values as reported by SNMP agent
    (objids, values) = instance.run (sys.argv[1], sys.argv[2], sys.argv[3:])

    # Convert two lists into a list of tuples for easier printing
    results = map (None, objids, values)

    # Convert two lists into a list of tuples for easier printing
    results = map (None, objids, values)

    # Just print them out
    for (objid, value) in results:
        print objid + ' ---> ' + str(value)
