#!/usr/local/bin/python -O
"""
   Set new value to MIB variables associated with user specifed
   SNMP Object IDs at arbitrary SNMP agent.

   Since MIB parser is not yet implemented in Python, this script takes
   and reports Object IDs in dotted numeric representation only.

   Written by Ilya Etingof <ilya@glas.net>, 10/2000

"""
import string

# Import PySNMP module
from pysnmp import session

class snmpset (session.session):
    """Set arbitrary variables at remote SNMP process
    """
    def __init__ (self, agent, community):
        """Explicitly call superclass's constructor as it gets
           overloaded by this class constructor and pass a few
           arguments alone.
        """   
        session.session.__init__ (self, agent, community)

    def run (self, objids, values):
        """Pass SNMP agent for one or more Object IDs. The objids
           argument should be a list of strings where each string
           represents a Object ID in dotted numbers notation (e.g.
           ['.1.3.6.1.4.1.307.3.2.1.1.1.4.1']).
        """   
        # Convert string type Object IDs into numeric representation
        numeric_objids = map (self.str2nums, objids)

        # BER encode SNMP Object IDs to query
        encoded_objids = map (self.encode_oid, numeric_objids)

        # BER encode the values of MIB variables (assuming they are strings!)
        encoded_values = map (self.encode_string, values)

        # Build a complete SNMP message of type 'SETREQUEST', pass it lists
        # of BER encoded Object IDs and MIB variables' values associated
        # with these Object IDs to set at SNMP agent.
        question = self.encode_request ('SETREQUEST', encoded_objids, encoded_values)

        # Try to send SNMP message to SNMP agent and receive a response.
        answer = self.send_and_receive (question)

        # As we get a response from SNMP agent, try to disassemble SNMP reply
        # and extract two lists of BER encoded SNMP Object IDs and 
        # associated values).
        (encoded_objids, encoded_values) = self.decode_response (answer)

        # Decode BER encoded Object ID.
        objids = map (self.decode_value, encoded_objids)

        # Decode BER encoded values associated with Object IDs.
        values = map (self.decode_value, encoded_values)

        # Return a tuple of two lists holding Object IDs and associated
        # values extracted from SNMP agent reply.
        return (objids, values)

if __name__ == '__main__':
    """Run the module if it's invoked for execution
    """
    import sys

    # Make sure we have got enough args
    if len (sys.argv) < 4:
        print 'Usage: %s <snmp-agent> <communuty> <object-id:value [[object-id:value] ... ]>' % sys.argv[0]
        sys.exit (1)

    # Initialize lists of passed Object-IDs and values
    objids = []
    values = []
    
    # Parse Object-IDs and values from command line arguments
    for arg in sys.argv[3:]:
        # Split token on Object-ID and value
        token = string.split(arg, ':')

        # Add them to the dictionary
        if len(token) == 2:
            # Add Object-ID and value into two lists
            objids.append (token[0])
            values.append (token[1])
        else:
            # Report a falure
            print 'bad argument: ', arg

            # Bad arguments, exiting
            sys.exit (1)
            
    # Create an instance of snmpset class
    instance = snmpset (sys.argv[1], sys.argv[2])

    # Run snmpset against passed Object IDs and expect it to return a list
    # of Object IDs and corresponding values as reported by SNMP agent
    (objids, values) = instance.run (objids, values)

    # Convert two lists into a list of tuples for easier printing
    results = map (None, objids, values)

    # Just print them out
    for (objid, value) in results:
        print objid + ' ---> ' + str(value)
